# Copyright (c) lobsterpy development team
# Distributed under the terms of a BSD 3-Clause "New" or "Revised" License

"""
This module defines classes to describe the COHPs automatically
"""
from __future__ import annotations

from pathlib import Path

from lobsterpy.plotting import PlainCohpPlotter


class Description:
    """
    Base class that will write generate a text description for all relevant bonds.
    It analyses all relevant coordination environments in the system based on electronic structure theory.

    """

    def __init__(self, analysis_object):
        """

        Args:
            analysis_object: Analysis object from lobsterpy.analysis
        """
        self.analysis_object = analysis_object
        self.set_description()

    def set_description(self):
        """
        This class will set the description of the structures correctly.
        Important is that here the naming of the cations from the lobster files will be used
        This means that the numbers will start at 1

        Returns:
            None

        """
        self.condensed_bonding_analysis = (
            self.analysis_object.condensed_bonding_analysis
        )
        if self.analysis_object.whichbonds == "cation-anion":
            relevant_cations = ", ".join(
                [
                    str(site.specie) + str(isite + 1)
                    for isite, site in enumerate(self.analysis_object.structure)
                    if isite in self.analysis_object.set_inequivalent_ions
                ]
            )
            self.text = []
            self.text.append(
                "The compound "
                + str(self.condensed_bonding_analysis["formula"])
                + " has "
                + str(self.condensed_bonding_analysis["number_of_considered_ions"])
                + " symmetry-independent cation(s) with relevant cation-anion interactions: "
                + str(relevant_cations)
                + "."
            )

            for key, item in self.condensed_bonding_analysis["sites"].items():
                # It has 3 Ta-N (mean ICOHP: -4.78 eV, antibonding interactions below EFermi),
                bond_info = []
                for type, properties in item["bonds"].items():
                    if not properties["has_antibdg_states_below_Efermi"]:
                        bond_info.append(
                            str(properties["number_of_bonds"])
                            + " "
                            + item["ion"]
                            + "-"
                            + str(type)
                            + " (mean ICOHP: "
                            ""
                            + properties["ICOHP_mean"]
                            + " eV, 0.0 percent antibonding interaction below EFermi)"
                        )
                    else:
                        bond_info.append(
                            str(properties["number_of_bonds"])
                            + " "
                            + item["ion"]
                            + "-"
                            + str(type)
                            + " (mean ICOHP: "
                            ""
                            + properties["ICOHP_mean"]
                            + " eV, "
                            + str(round(properties["antibonding"]["perc"] * 100, 3))
                            + " percent antibonding interaction below EFermi)"
                        )

                if len(bond_info) > 1:
                    bonds = ",".join(bond_info[0:-1]) + ", and " + bond_info[-1]
                else:
                    bonds = bond_info[0]
                if item["env"] == "O:6":
                    self.text.append(
                        str(item["ion"])
                        + str(key + 1)
                        + " has an "
                        + str(self._coordination_environment_to_text(item["env"]))
                        + " coordination environment. It has "
                        + str(bonds)
                        + " bonds."
                    )
                else:
                    self.text.append(
                        str(item["ion"])
                        + str(key + 1)
                        + " has a "
                        + str(self._coordination_environment_to_text(item["env"]))
                        + " coordination environment. It has "
                        + str(bonds)
                        + " bonds."
                    )
        elif self.analysis_object.whichbonds == "all":
            relevant_ions = ", ".join(
                [
                    str(site.specie) + str(isite + 1)
                    for isite, site in enumerate(self.analysis_object.structure)
                    if isite in self.analysis_object.set_inequivalent_ions
                ]
            )
            self.text = []
            self.text.append(
                "The compound "
                + str(self.condensed_bonding_analysis["formula"])
                + " has "
                + str(self.condensed_bonding_analysis["number_of_considered_ions"])
                + " symmetry-independent atoms(s) with relevant bonds: "
                + str(relevant_ions)
                + "."
            )

            for key, item in self.condensed_bonding_analysis["sites"].items():
                # It has 3 Ta-N (mean ICOHP: -4.78 eV, antibonding interactions below EFermi),
                bond_info = []
                for type, properties in item["bonds"].items():
                    if not properties["has_antibdg_states_below_Efermi"]:
                        bond_info.append(
                            str(properties["number_of_bonds"])
                            + " "
                            + item["ion"]
                            + "-"
                            + str(type)
                            + " (mean ICOHP: "
                            ""
                            + properties["ICOHP_mean"]
                            + " eV, 0.0 percent antibonding interaction below EFermi)"
                        )
                    else:
                        bond_info.append(
                            str(properties["number_of_bonds"])
                            + " "
                            + item["ion"]
                            + "-"
                            + str(type)
                            + " (mean ICOHP: "
                            ""
                            + properties["ICOHP_mean"]
                            + " eV, "
                            + str(round(properties["antibonding"]["perc"] * 100, 3))
                            + " percent antibonding interaction below EFermi)"
                        )

                if len(bond_info) > 1:
                    bonds = ",".join(bond_info[0:-1]) + ", and " + bond_info[-1]
                else:
                    bonds = bond_info[0]
                if item["env"] == "O:6":
                    self.text.append(
                        str(item["ion"])
                        + str(key + 1)
                        + " has an "
                        + str(self._coordination_environment_to_text(item["env"]))
                        + " coordination environment. It has "
                        + str(bonds)
                        + " bonds."
                    )
                else:
                    self.text.append(
                        str(item["ion"])
                        + str(key + 1)
                        + " has a "
                        + str(self._coordination_environment_to_text(item["env"]))
                        + " coordination environment. It has "
                        + str(bonds)
                        + " bonds."
                    )

        if "madelung_energy" in self.analysis_object.condensed_bonding_analysis:
            self.text.append(
                "The Madelung energy of this crystal structure per unit cell is: "
                + str(
                    self.analysis_object.condensed_bonding_analysis["madelung_energy"]
                )
                + " eV."
            )

    def plot_cohps(
        self,
        save=False,
        filename=None,
        ylim=[-4, 2],
        xlim=None,
        integrated=False,
        title="",
        sigma=None,
        skip_show=False,
    ):
        """
        Will automatically generate plots of the most relevant COHP

        Args:
            save (bool): will save the plot to a file
            filename (str/Path):
            ylim (list of float): energy scale that is shown in plot (eV)
            xlim(list of float): energy range for COHPs in eV
            integrated (bool): if True, integrated COHPs will be shown
            sigma: Standard deviation of Gaussian broadening applied to
                population data. If None, no broadening will be added.
            skip_show (bool): if True, the plot will not be shown.

        Returns:
            A matplotlib object.

        """
        set_cohps = self.analysis_object.set_cohps
        if self.analysis_object.whichbonds == "cation-anion":
            set_inequivalent_cations = self.analysis_object.set_inequivalent_ions
        elif self.analysis_object.whichbonds == "all":
            set_inequivalent_cations = self.analysis_object.set_inequivalent_ions
        set_labels_cohps = self.analysis_object.set_labels_cohps
        structure = self.analysis_object.structure

        for iplot, (ication, labels, cohps) in enumerate(
            zip(set_inequivalent_cations, set_labels_cohps, set_cohps)
        ):
            namecation = str(structure[ication].specie)

            cp = PlainCohpPlotter()
            for label, cohp in zip(labels, cohps):
                if label is not None:
                    cp.add_cohp(namecation + str(ication + 1) + ": " + label, cohp)
            plot = cp.get_plot(integrated=integrated, sigma=sigma)
            plot.ylim(ylim)
            if xlim is not None:
                plot.xlim(xlim)

            plot.title(title)
            if save:
                if len(set_inequivalent_cations) > 1:
                    if isinstance(filename, str):
                        filename = Path(filename)
                    filename_new = (
                        filename.parent / f"{filename.stem}-{iplot}{filename.suffix}"
                    )
                else:
                    filename_new = filename
                plot.savefig(filename_new)
        if not skip_show:
            plot.show()
        else:
            plot.close()

    @staticmethod
    def _coordination_environment_to_text(ce):
        """
        This method transfers a coordination environment str into a text description of the environment

        Args:
            ce (str): output from ChemEnv package (e.g., "O:6")

        Returns:
            A text description of coordination environment
        """
        if ce == "S:1":
            return "single (CN=1)"
        if ce == "L:2":
            return "linear (CN=2)"
        if ce == "A:2":
            return "angular (CN=2)"
        if ce == "TL:3":
            return "trigonal planar (CN=3)"
        if ce == "TY:3":
            return "triangular non-coplanar (CN=3)"
        if ce == "TS:3":
            return "t-shaped (CN=3)"
        if ce == "T:4":
            return "tetrahedral (CN=4)"
        if ce == "S:4":
            return "square planar (CN=4)"
        if ce == "SY:4":
            return "square non-coplanar (CN=4)"
        if ce == "SS:4":
            return "see-saw like (CN=4)"
        if ce == "PP:5":
            return "pentagonal (CN=5)"
        if ce == "S:5":
            return "square pyramidal (CN=5)"
        if ce == "T:5":
            return "trigonal bipyramidal (CN=5)"
        if ce == "O:6":
            return "octahedral (CN=6)"
        if ce == "T:6":
            return "trigonal prismatic (CN=6)"
        if ce == "PP:6":
            return "pentagonal pyramidal (CN=6)"
        if ce == "PB:7":
            return "pentagonal bipyramidal (CN=7)"
        if ce == "ST:7":
            return "square-face capped trigonal prismatic (CN=7)"
        if ce == "ET:7":
            return "end-trigonal-face capped trigonal prismatic (CN=7)"
        if ce == "FO:7":
            return "face-capped octahedron (CN=7)"
        if ce == "C:8":
            return "cubic (CN=8)"
        if ce == "SA:8":
            return "square antiprismatic (CN=8)"
        if ce == "SBT:8":
            return "square-face bicapped trigonal prismatic (CN=8)"
        if ce == "TBT:8":
            return "triangular-face bicapped trigonal prismatic (CN=8)"
        if ce == "DD:8":
            return "dodecahedronal (with triangular faces) (CN=8)"
        if ce == "DDPN:8":
            return (
                "dodecahedronal (with triangular faces - p2345 plane normalized) (CN=8)"
            )
        if ce == "HB:8":
            return "hexagonal bipyramidal (CN=8)"
        if ce == "BO_1:8":
            return "bicapped octahedral (opposed cap faces) (CN=8)"
        if ce == "BO_2:8":
            return "bicapped octahedral (cap faces with one atom in common) (CN=8)"
        if ce == "BO_3:8":
            return "bicapped octahedral (cap faces with one edge in common) (CN=8)"
        if ce == "TC:9":
            return "triangular cupola (CN=9)"
        if ce == "TT_1:9":
            return "Tricapped triangular prismatic (three square - face caps) (CN=9)"
        if ce == "TT_2:9":
            return "Tricapped triangular prismatic (two square - face caps and one triangular - face cap) (CN=9)"
        if ce == "TT_3:9":
            return "Tricapped triangular prism (one square - face cap and two triangular - face caps) (CN=9)"
        if ce == "HD:9":
            return "Heptagonal dipyramidal (CN=9)"
        if ce == "TI:9":
            return "tridiminished icosohedral (CN=9)"
        if ce == "SMA:9":
            return "Square-face monocapped antiprism (CN=9)"
        if ce == "SS:9":
            return "Square-face capped square prismatic (CN=9)"
        if ce == "TO_1:9":
            return "Tricapped octahedral (all 3 cap faces share one atom) (CN=9)"
        if ce == "TO_2:9":
            return "Tricapped octahedral (cap faces are aligned) (CN=9)"
        if ce == "TO_3:9":
            return "Tricapped octahedron (all 3 cap faces are sharing one edge of a face) (CN=9)"
        if ce == "PP:10":
            return "Pentagonal prismatic (CN=10)"
        if ce == "PA:10":
            return "Pentagonal antiprismatic (CN=10)"
        if ce == "SBSA:10":
            return "Square-face bicapped square antiprismatic (CN=10)"
        if ce == "MI:10":
            return "Metabidiminished icosahedral (CN=10)"
        if ce == "S:10":
            return "sphenocoronal (CN=10)"
        if ce == "H:10":
            return "Hexadecahedral (CN=10)"
        if ce == "BS_1:10":
            return "Bicapped square prismatic (opposite faces) (CN=10)"
        if ce == "BS_1:10":
            return "Bicapped square prismatic (opposite faces) (CN=10)"
        if ce == "BS_2:10":
            return "Bicapped square prism(adjacent faces) (CN=10)"
        if ce == "TBSA:10":
            return "Trigonal-face bicapped square antiprismatic (CN=10)"
        if ce == "PCPA:11":
            return "Pentagonal - face capped pentagonal antiprismatic (CN=11)"
        if ce == "H:11":
            return "Hendecahedral (CN=11)"
        if ce == "SH:11":
            return "Sphenoid hendecahedral (CN=11)"
        if ce == "CO:11":
            return "Cs - octahedral (CN=11)"
        if ce == "DI:11":
            return "Diminished icosahedral (CN=12)"
        if ce == "I:12":
            return "Icosahedral (CN=12)"
        if ce == "PBP: 12":
            return "Pentagonal - face bicapped pentagonal prismatic (CN=12)"
        if ce == "TT:12":
            return "Truncated tetrahedral (CN=12)"
        if ce == "C:12":
            return "Cuboctahedral (CN=12)"
        if ce == "AC:12":
            return "Anticuboctahedral (CN=12)"
        if ce == "SC:12":
            return "Square cupola (CN=12)"
        if ce == "S:12":
            return "Sphenomegacorona (CN=12)"
        if ce == "HP:12":
            return "Hexagonal prismatic (CN=12)"
        if ce == "HA:12":
            return "Hexagonal antiprismatic (CN=12)"
        if ce == "SH:13":
            return "Square-face capped hexagonal prismatic (CN=13)"
        if ce == "1":
            return "1-fold"
        if ce == "2":
            return "2-fold"
        if ce == "3":
            return "3-fold"
        if ce == "4":
            return "4-fold"
        if ce == "5":
            return "5-fold"
        if ce == "6":
            return "6-fold"
        if ce == "7":
            return "7-fold"
        if ce == "8":
            return "8-fold"
        if ce == "9":
            return "9-fold"
        if ce == "10":
            return "10-fold"
        if ce == "11":
            return "11-fold"
        if ce == "12":
            return "12-fold"
        if ce == "13":
            return "13-fold"
        if ce == "14":
            return "14-fold"
        if ce == "15":
            return "15-fold"
        if ce == "16":
            return "16-fold"
        if ce == "17":
            return "17-fold"
        if ce == "18":
            return "18-fold"
        if ce == "19":
            return "19-fold"
        if ce == "20":
            return "20-fold"
        if ce == "21":
            return "21-fold"
        if ce == "22":
            return "22-fold"
        if ce == "23":
            return "23-fold"
        if ce == "24":
            return "24-fold"
        if ce == "25":
            return "25-fold"
        if ce == "26":
            return "26-fold"
        if ce == "27":
            return "27-fold"
        if ce == "28":
            return "28-fold"
        if ce == "29":
            return "29-fold"
        if ce == "30":
            return "30-fold"
        return ce

    def write_description(self):
        """
        This method will print the description of the COHPs to the screen

        """
        for textpart in self.text:
            print(textpart)



from __future__ import annotations

import io
import json
import os
from contextlib import redirect_stdout
from pathlib import Path

import matplotlib.pyplot as plt
import matplotlib.style
import numpy as np
import pytest
from matplotlib.figure import Figure

from lobsterpy.cli import get_parser, run

CurrentDir = Path(__file__).absolute().parent
TestDir = CurrentDir / "../"
ref_data_file = TestDir / "TestData/cli-reference.json"
test_cases = [
    ["automatic-plot"],
    ["automaticplot", "--allbonds"],
    ["automatic-plot", "--style", "ggplot"],
    ["automatic-plot", "--sigma", "1.2"],
    ["description"],
    ["description", "--all-bonds"],
    ["plot", "1", "2"],
    ["plot", "1", "--cobis"],
    ["plot", "1", "--coops"],
    ["plot", "--summed", "1", "2"],
    ["plot", "1", "2", "--orbitalwise", "3s-3s", "3s-3s"],
    ["plot", "1", "--orbitalwise", "all"],
    ["plot", "1", "--fontsize", "20"],
    ["plot", "1", "--width", "20", "--height", "20"],
    ["plot", "1", "--width", "20"],
    ["plot", "1", "--height", "20"],
    ["plot", "1", "--style", "dark_background"],
    ["plot", "1", "--sigma", "1.2"],
    ["plot", "1", "--fwhm", "1"],
]

error_test_cases = [
    (["plot", "1", "2", "--orbitalwise", "3s-3s"], IndexError),
    (["plot", "400000"], IndexError),
    (["plot", "1", "--orbitalwise", "1s-1s"], IndexError),
]


class TestCLI:
    @pytest.fixture
    def inject_mocks(self, mocker):
        # Disable calls to show() so we can get the current figure using gcf()
        mocker.patch("matplotlib.pyplot.show")
        mocker.resetall()

    @pytest.fixture
    def clean_plot(self):
        yield
        plt.close("all")
        matplotlib.style.use("default")

    with open(ref_data_file) as fd:
        ref_results = json.load(fd)

    @classmethod
    def setup_class(cls):
        os.chdir(TestDir / "TestData/NaCl")

    @pytest.mark.parametrize("args", test_cases)
    def test_cli_results(self, args, capsys, inject_mocks, clean_plot):
        test = get_parser().parse_args(args)
        run(test)

        captured = capsys.readouterr()
        assert captured.out == self.ref_results[" ".join(args)]["stdout"]

        plot_attributes = self.get_plot_attributes(plt.gcf())
        ref_plot_attributes = self.ref_results[" ".join(args)]["plot"]

        if ref_plot_attributes is None:
            assert plot_attributes is None

        else:
            for key, ref_value in ref_plot_attributes.items():
                if key == "xydata":
                    for line, ref_line in zip(plot_attributes[key], ref_value):
                        assert np.array(np.array(line)) == pytest.approx(
                            np.array(ref_line)
                        )
                else:
                    assert plot_attributes[key] == pytest.approx(ref_value)

    @pytest.mark.parametrize("args, error", error_test_cases)
    def test_cli_errors(self, args, error, inject_mocks):
        with pytest.raises(error):
            test = get_parser().parse_args(args)
            run(test)

    def test_plot_saved(self, tmp_path, inject_mocks, clean_plot):
        plot_path = tmp_path / "plot.png"
        args = ["plot", "1", "--saveplot", str(plot_path)]
        test = get_parser().parse_args(args)
        run(test)
        self.assert_is_finite_file(plot_path)

    def test_json_saved(self, tmp_path, inject_mocks, clean_plot):
        json_path = tmp_path / "data.json"
        args = ["automatic-plot", "--json", str(json_path)]
        test = get_parser().parse_args(args)
        run(test)
        self.assert_is_finite_file(json_path)

    def test_lobsterin_generation(self, tmp_path):
        os.chdir(TestDir / "TestData/Test_Input_Generation_Empty")
        lobsterinpath = tmp_path / "lobsterin.lobsterpy"
        INCARpath = tmp_path / "INCAR.lobsterpy"
        args = [
            "create-inputs",
            "--lobsterin-out",
            str(lobsterinpath),
            "--incar-out",
            str(INCARpath),
        ]
        test = get_parser().parse_args(args)
        run(test)
        for filepath in [
            tmp_path / "lobsterin.lobsterpy-0",
            tmp_path / "lobsterin.lobsterpy-1",
            tmp_path / "INCAR.lobsterpy-0",
            tmp_path / "INCAR.lobsterpy-1",
        ]:
            self.assert_is_finite_file(filepath)

        args = [
            "create-inputs",
            "--lobsterin-out",
            str(lobsterinpath),
            "--incar-out",
            str(INCARpath),
            "--overwrite",
        ]
        test = get_parser().parse_args(args)
        run(test)

        os.chdir(TestDir / "TestData/NaCl")

    def test_lobsterin_generation_error(self, tmp_path):
        os.chdir(TestDir / "TestData/Test_Input_Generation_Empty")
        lobsterinpath = tmp_path / "lobsterin.lobsterpy"
        INCARpath = tmp_path / "INCAR.lobsterpy"
        args = [
            "create-inputs",
            "--lobsterin-out",
            str(lobsterinpath),
            "--incar-out",
            str(INCARpath),
        ]
        test = get_parser().parse_args(args)
        run(test)
        for filepath in [
            tmp_path / "lobsterin.lobsterpy-0",
            tmp_path / "lobsterin.lobsterpy-1",
            tmp_path / "INCAR.lobsterpy-0",
            tmp_path / "INCAR.lobsterpy-1",
        ]:
            self.assert_is_finite_file(filepath)

        args = [
            "create-inputs",
            "--lobsterin-out",
            str(lobsterinpath),
            "--incar-out",
            str(INCARpath),
        ]
        test = get_parser().parse_args(args)
        with pytest.raises(ValueError):
            run(test)
        os.chdir(TestDir / "TestData/NaCl")

    def test_lobsterin_generation_error_userbasis(self, tmp_path):
        # This is a test for the user-defined basis set.
        os.chdir(TestDir / "TestData/Test_Input_Generation_Empty")
        lobsterinpath = tmp_path / "lobsterin.lobsterpy"
        INCARpath = tmp_path / "INCAR.lobsterpy"
        args = [
            "create-inputs",
            "--lobsterin-out",
            str(lobsterinpath),
            "--incar-out",
            str(INCARpath),
            "--userbasis",
            "Na.3s.3p Cl.3s.3p",
        ]
        test = get_parser().parse_args(args)
        run(test)

        for basis in ["Na 3s 3p", "Cl 3s 3p"]:
            assert basis in open(tmp_path / "lobsterin.lobsterpy-0").read()

        for filepath in [
            tmp_path / "lobsterin.lobsterpy-0",
            tmp_path / "INCAR.lobsterpy-0",
        ]:
            self.assert_is_finite_file(filepath)

        os.chdir(TestDir / "TestData/NaCl")

    @staticmethod
    def assert_is_finite_file(path: Path) -> None:
        assert path.is_file()
        assert path.stat().st_size > 0

    @pytest.mark.skip(reason="Only enable this test to regenerate test data")
    def test_generate_ref_data(self, inject_mocks):
        json_data = {}

        for args in test_cases:
            plt.close("all")
            matplotlib.style.use("default")

            with redirect_stdout(io.StringIO()) as stdout:
                test = get_parser().parse_args(args)
                run(test)
                json_data[" ".join(args)] = {"stdout": stdout.getvalue()}

                fig = plt.gcf()
                json_data[" ".join(args)].update(
                    {"plot": self.get_plot_attributes(fig)}
                )

        with open(ref_data_file, "w") as fd:
            json.dump(json_data, fd, indent=4, sort_keys=True)

    @staticmethod
    def get_plot_attributes(fig: Figure) -> dict | None:
        if fig.axes:
            ax = fig.gca()

            return {
                "xydata": [line.get_xydata().tolist() for line in ax.lines],
                "facecolor": ax.get_facecolor(),
                "size": fig.get_size_inches().tolist(),
            }
        else:
            return None

    def teardown_class(self) -> None:
        os.chdir(CurrentDir)



from __future__ import annotations

import unittest
from pathlib import Path

from lobsterpy.cohp.analyze import Analysis
from lobsterpy.cohp.describe import Description

CurrentDir = Path(__file__).absolute().parent
TestDir = CurrentDir / "../../"


# TODO: Add example without antibonding states


class TestDescribe(unittest.TestCase):
    def setUp(self):
        self.analyse_NaCl = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )
        self.describe_NaCl = Description(self.analyse_NaCl)

        self.analyse_NaCl_valences = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=None,
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )
        self.describe_NaCl_valences = Description(self.analyse_NaCl_valences)

        self.analyse_BaTiO3 = Analysis(
            path_to_poscar=TestDir / "TestData/BaTiO3/POSCAR",
            path_to_cohpcar=TestDir / "TestData/BaTiO3/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/BaTiO3/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/BaTiO3/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.describe_BaTiO3 = Description(self.analyse_BaTiO3)

        self.analyse_BaTaO2N1 = Analysis(
            path_to_poscar=TestDir / "TestData/BaTaO2N1/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/BaTaO2N1/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/BaTaO2N1/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/BaTaO2N1/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )
        self.describe_BaTaO2N1 = Description(self.analyse_BaTaO2N1)

        self.describe_BaTiO3 = Description(self.analyse_BaTiO3)

        self.analyse_CdF = Analysis(
            path_to_poscar=TestDir / "TestData/CdF/POSCAR",
            path_to_cohpcar=TestDir / "TestData/CdF/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/CdF/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/CdF/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )
        self.describe_CdF = Description(self.analyse_CdF)

        self.analyse_NaCl_distorted = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl_distorted/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl_distorted/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl_distorted/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl_distorted/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.describe_NaCl_distorted = Description(self.analyse_NaCl_distorted)

        self.analyse_NaCl_spin = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl_spin/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl_spin/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl_spin/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl_spin/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            summed_spins=False,
        )
        self.describe_NaCl_spin = Description(self.analyse_NaCl_spin)

        self.analyse_NaCl_all = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            whichbonds="all",
            cutoff_icohp=0.1,
        )

        self.describe_Nacl_all = Description(self.analyse_NaCl_all)

        self.analyse_NaCl_madelung_all = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            path_to_madelung=TestDir / "TestData/NaCl/MadelungEnergies.lobster",
            whichbonds="all",
            cutoff_icohp=0.1,
        )

        self.describe_Nacl_madelung_all = Description(self.analyse_NaCl_madelung_all)

        self.analyse_NaSi_madelung_all = Analysis(
            path_to_poscar=TestDir / "TestData/NaSi/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaSi/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaSi/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaSi/CHARGE.lobster",
            path_to_madelung=TestDir / "TestData/NaSi/MadelungEnergies.lobster",
            whichbonds="all",
            cutoff_icohp=0.1,
        )

        self.describe_NaSi_madelung_all = Description(self.analyse_NaSi_madelung_all)

        self.analyse_NaSbF6 = Analysis(
            path_to_poscar=TestDir / "TestData/NaSbF6/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/NaSbF6/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/NaSbF6/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/NaSbF6/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )
        self.describe_NaSbF6 = Description(self.analyse_NaSbF6)

        self.analyse_NaSbF6_anbd = Analysis(
            path_to_poscar=TestDir / "TestData/NaSbF6/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/NaSbF6/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/NaSbF6/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/NaSbF6/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            start=-5.5,
        )

        self.describe_NaSbF6_anbd = Description(self.analyse_NaSbF6_anbd)

        self.analyse_NaCl_nan = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            start=-4.0,
        )

        self.describe_NaCl_nan = Description(self.analyse_NaCl_nan)

        self.analyse_CdF_anbd = Analysis(
            path_to_poscar=TestDir / "TestData/CdF/POSCAR",
            path_to_cohpcar=TestDir / "TestData/CdF/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/CdF/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/CdF/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            start=-4.0,
        )

        self.describe_CdF_anbd = Description(self.analyse_CdF_anbd)

    def test_coordination_environment_to_text(self):
        results_dict = {
            "S:1": "single (CN=1)",
            "L:2": "linear (CN=2)",
            "A:2": "angular (CN=2)",
            "TL:3": "trigonal planar (CN=3)",
            "TY:3": "triangular non-coplanar (CN=3)",
            "TS:3": "t-shaped (CN=3)",
            "T:4": "tetrahedral (CN=4)",
            "S:4": "square planar (CN=4)",
            "SY:4": "square non-coplanar (CN=4)",
            "SS:4": "see-saw like (CN=4)",
            "PP:5": "pentagonal (CN=5)",
            "S:5": "square pyramidal (CN=5)",
            "T:5": "trigonal bipyramidal (CN=5)",
            "O:6": "octahedral (CN=6)",
            "T:6": "trigonal prismatic (CN=6)",
            "PP:6": "pentagonal pyramidal (CN=6)",
            "PB:7": "pentagonal bipyramidal (CN=7)",
            "ST:7": "square-face capped trigonal prismatic (CN=7)",
            "ET:7": "end-trigonal-face capped trigonal prismatic (CN=7)",
            "FO:7": "face-capped octahedron (CN=7)",
            "C:8": "cubic (CN=8)",
            "SA:8": "square antiprismatic (CN=8)",
            "SBT:8": "square-face bicapped trigonal prismatic (CN=8)",
            "TBT:8": "triangular-face bicapped trigonal prismatic (CN=8)",
            "DD:8": "dodecahedronal (with triangular faces) (CN=8)",
            "DDPN:8": "dodecahedronal (with triangular faces - p2345 plane normalized) (CN=8)",
            "HB:8": "hexagonal bipyramidal (CN=8)",
            "BO_1:8": "bicapped octahedral (opposed cap faces) (CN=8)",
            "BO_2:8": "bicapped octahedral (cap faces with one atom in common) (CN=8)",
            "BO_3:8": "bicapped octahedral (cap faces with one edge in common) (CN=8)",
            "TC:9": "triangular cupola (CN=9)",
            "TT_1:9": "Tricapped triangular prismatic (three square - face caps) (CN=9)",
            "TT_2:9": "Tricapped triangular prismatic (two square - face caps and one triangular - face cap) (CN=9)",
            "TT_3:9": "Tricapped triangular prism (one square - face cap and two triangular - face caps) (CN=9)",
            "HD:9": "Heptagonal dipyramidal (CN=9)",
            "TI:9": "tridiminished icosohedral (CN=9)",
            "SMA:9": "Square-face monocapped antiprism (CN=9)",
            "SS:9": "Square-face capped square prismatic (CN=9)",
            "TO_1:9": "Tricapped octahedral (all 3 cap faces share one atom) (CN=9)",
            "TO_2:9": "Tricapped octahedral (cap faces are aligned) (CN=9)",
            "TO_3:9": "Tricapped octahedron (all 3 cap faces are sharing one edge of a face) (CN=9)",
            "PP:10": "Pentagonal prismatic (CN=10)",
            "PA:10": "Pentagonal antiprismatic (CN=10)",
            "SBSA:10": "Square-face bicapped square antiprismatic (CN=10)",
            "MI:10": "Metabidiminished icosahedral (CN=10)",
            "S:10": "sphenocoronal (CN=10)",
            "H:10": "Hexadecahedral (CN=10)",
            "BS_1:10": "Bicapped square prismatic (opposite faces) (CN=10)",
            "BS_2:10": "Bicapped square prism(adjacent faces) (CN=10)",
            "TBSA:10": "Trigonal-face bicapped square antiprismatic (CN=10)",
            "PCPA:11": "Pentagonal - face capped pentagonal antiprismatic (CN=11)",
            "H:11": "Hendecahedral (CN=11)",
            "SH:11": "Sphenoid hendecahedral (CN=11)",
            "CO:11": "Cs - octahedral (CN=11)",
            "DI:11": "Diminished icosahedral (CN=12)",
            "I:12": "Icosahedral (CN=12)",
            "PBP: 12": "Pentagonal - face bicapped pentagonal prismatic (CN=12)",
            "TT:12": "Truncated tetrahedral (CN=12)",
            "C:12": "Cuboctahedral (CN=12)",
            "AC:12": "Anticuboctahedral (CN=12)",
            "SC:12": "Square cupola (CN=12)",
            "S:12": "Sphenomegacorona (CN=12)",
            "HP:12": "Hexagonal prismatic (CN=12)",
            "HA:12": "Hexagonal antiprismatic (CN=12)",
            "SH:13": "Square-face capped hexagonal prismatic (CN=13)",
            "H:5": "H:5",
            "1": "1-fold",
            "2": "2-fold",
            "3": "3-fold",
            "4": "4-fold",
            "5": "5-fold",
            "6": "6-fold",
            "7": "7-fold",
            "8": "8-fold",
            "9": "9-fold",
            "10": "10-fold",
            "11": "11-fold",
            "12": "12-fold",
            "13": "13-fold",
            "14": "14-fold",
            "15": "15-fold",
            "16": "16-fold",
            "17": "17-fold",
            "18": "18-fold",
            "19": "19-fold",
            "20": "20-fold",
            "21": "21-fold",
            "22": "22-fold",
            "23": "23-fold",
            "24": "24-fold",
            "25": "25-fold",
            "26": "26-fold",
            "27": "27-fold",
            "28": "28-fold",
            "29": "29-fold",
            "30": "30-fold",
        }
        for key, items in results_dict.items():
            self.assertEqual(Description._coordination_environment_to_text(key), items)

    def test_plot(self):
        import tempfile

        with tempfile.TemporaryDirectory() as tmp0:
            filename_test = Path(tmp0) / "test.pdf"
            self.describe_NaCl.plot_cohps(
                save=True, filename=filename_test, xlim=[-4, 4]
            )
            self.assertTrue(Path(filename_test).exists())

        with tempfile.TemporaryDirectory() as tmp1:
            filename_test = Path(tmp1) / "test.pdf"
            self.describe_NaCl_spin.plot_cohps(
                save=True, filename=filename_test, xlim=[-4, 4]
            )
            self.assertTrue(Path(filename_test).exists())

        with tempfile.TemporaryDirectory() as tmp2:
            filename_test = Path(tmp2) / "test.pdf"
            self.describe_Nacl_all.plot_cohps(
                save=True, filename=filename_test, xlim=[-4, 4]
            )
            filename_test_1 = Path(tmp2) / "test-0.pdf"
            filename_test_2 = Path(tmp2) / "test-1.pdf"
            self.assertFalse(Path(filename_test).exists())
            self.assertTrue(Path(filename_test_1).exists())
            self.assertTrue(Path(filename_test_2).exists())

        with tempfile.TemporaryDirectory() as tmp2:
            filename_test = str(Path(tmp2) / "test.pdf")
            self.describe_Nacl_all.plot_cohps(
                save=True, filename=filename_test, xlim=[-4, 4]
            )
            filename_test_1 = Path(tmp2) / "test-0.pdf"
            filename_test_2 = Path(tmp2) / "test-1.pdf"
            self.assertFalse(Path(filename_test).exists())
            self.assertTrue(Path(filename_test_1).exists())
            self.assertTrue(Path(filename_test_2).exists())

    def test_write_descritoin(self):
        self.describe_NaCl.write_description()
        self.describe_NaSi_madelung_all.write_description()
        self.describe_NaSbF6.write_description()
        self.describe_NaSbF6_anbd.write_description()
        self.describe_CdF_anbd.write_description()
        self.describe_NaCl_nan.write_description()

    def test_text(self):
        self.assertEqual(
            self.describe_CdF.text,
            [
                "The compound CdF2 has 1 symmetry-independent cation(s) with relevant cation-anion interactions: Cd1.",
                "Cd1 has a cubic (CN=8) coordination environment. It has 8 Cd-F (mean ICOHP: -0.62 eV, 44.26 percent antibonding interaction below EFermi) bonds.",
            ],
        )
        self.assertEqual(
            self.describe_NaCl.text,
            [
                "The compound NaCl has 1 symmetry-independent cation(s) with relevant cation-anion interactions: Na1.",
                "Na1 has an octahedral (CN=6) coordination environment. It has 6 Na-Cl (mean ICOHP: -0.57 eV, 3.448 percent antibonding interaction below EFermi) bonds.",
            ],
        )
        self.assertEqual(
            self.describe_NaCl.text,
            [
                "The compound NaCl has 1 symmetry-independent cation(s) with relevant cation-anion interactions: Na1.",
                "Na1 has an octahedral (CN=6) coordination environment. It has 6 Na-Cl (mean ICOHP: -0.57 eV, 3.448 percent antibonding interaction below EFermi) bonds.",
            ],
        )
        self.assertEqual(
            self.describe_NaSbF6.text,
            [
                "The compound NaSbF6 has 2 symmetry-independent cation(s) with relevant cation-anion interactions: Na1, Sb2.",
                "Na1 has an octahedral (CN=6) coordination environment. It has 6 Na-F (mean ICOHP: -0.61 eV, 4.071 percent antibonding interaction below EFermi) bonds.",
                "Sb2 has an octahedral (CN=6) coordination environment. It has 6 Sb-F (mean ICOHP: -5.45 eV, 0.0 percent antibonding interaction below EFermi) bonds.",
            ],
        )
        self.assertEqual(
            self.describe_NaSbF6_anbd.text,
            [
                "The compound NaSbF6 has 2 symmetry-independent cation(s) with relevant cation-anion interactions: Na1, Sb2.",
                "Na1 has an octahedral (CN=6) coordination environment. It has 6 Na-F (mean ICOHP: -0.61 eV, 0.0 percent antibonding interaction below EFermi) bonds.",
                "Sb2 has an octahedral (CN=6) coordination environment. It has 6 Sb-F (mean ICOHP: -5.45 eV, 0.0 percent antibonding interaction below EFermi) bonds.",
            ],
        )
        self.assertEqual(
            self.describe_NaCl_nan.text,
            [
                "The compound NaCl has 1 symmetry-independent cation(s) with relevant cation-anion interactions: Na1.",
                "Na1 has an octahedral (CN=6) coordination environment. It has 6 Na-Cl (mean ICOHP: -0.57 eV, 0.0 percent antibonding interaction below EFermi) bonds.",
            ],
        )
        self.assertEqual(
            self.describe_CdF_anbd.text,
            [
                "The compound CdF2 has 1 symmetry-independent cation(s) with relevant cation-anion interactions: Cd1.",
                "Cd1 has a cubic (CN=8) coordination environment. It has 8 Cd-F (mean ICOHP: -0.62 eV, 100.0 percent antibonding interaction below EFermi) bonds.",
            ],
        )


if __name__ == "__main__":
    unittest.main()



# Copyright (c) lobsterpy development team
# Distributed under the terms of a BSD 3-Clause "New" or "Revised" License

"""
Here classes and functions to plot Lobster outputs are provided
"""
from __future__ import annotations

from typing import Any, Dict, List, Optional, Tuple, Union

import matplotlib
import numpy as np
from matplotlib import pyplot as plt
from pkg_resources import resource_filename
from pymatgen.electronic_structure.core import Spin
from pymatgen.electronic_structure.plotter import CohpPlotter

base_style = resource_filename("lobsterpy.plotting", "lobsterpy_base.mplstyle")


def get_style_list(
    no_base_style: bool = False,
    styles: list[str | dict[str, Any]] | None = None,
    **kwargs,
) -> list[str | dict[str, Any]]:
    """Get *args for matplotlib.style from user input

    Args:
        no_base_style: If true, do not include lobsterpy_base.mplstyle
        styles: User-requested styles. These can be paths to mplstyle files,
                the names of known (matplotlib-supplied) styles,
                or dicts of rcParam options.

    Remaining kwargs are collected as a dict and take highest priority.
    """
    if no_base_style:
        base = []  # type: List[Union[str, Dict[str, Any]]]
    else:
        base = [base_style]

    if styles is None:
        styles = []

    return base + styles + [kwargs]


class PlainCohpPlotter(CohpPlotter):
    """
    Modified Pymatgen CohpPlotter with styling removed

    This allows the styling to be manipulated more easily using matplotlib
    style sheets.
    """

    def get_plot(
        self,
        ax: matplotlib.axes.Axes | None = None,
        xlim: tuple[float, float] | None = None,
        ylim: tuple[float, float] | None = None,
        plot_negative: bool | None = None,
        integrated: bool = False,
        invert_axes: bool = True,
        sigma: float | None = None,
    ):
        """
        Get a matplotlib plot showing the COHP.

        Args:
            ax: Existing Matplotlib Axes object to plot to.
            xlim: Specifies the x-axis limits. Defaults to None for
                automatic determination.
            ylim: Specifies the y-axis limits. Defaults to None for
                automatic determination.
            plot_negative: It is common to plot -COHP(E) so that the
                sign means the same for COOPs and COHPs. Defaults to None
                for automatic determination: If are_coops is True, this
                will be set to False, else it will be set to True.
            integrated: Switch to plot ICOHPs. Defaults to False.
            invert_axes: Put the energies onto the y-axis, which is
                common in chemistry.
            sigma: Standard deviation of Gaussian broadening applied to
                population data. If this is unset (None) no broadening will be
                added.

        Returns:
            A matplotlib object.
        """
        if self.are_coops:
            cohp_label = "COOP"
        elif self.are_cobis:
            cohp_label = "COBI"
        else:
            cohp_label = "COHP"

        if plot_negative is None:
            plot_negative = (not self.are_coops) and (not self.are_cobis)

        if integrated:
            cohp_label = "I" + cohp_label + " (eV)"

        if plot_negative:
            cohp_label = "$-$" + cohp_label

        if self.zero_at_efermi:
            energy_label = "$E - E_f$ (eV)"
        else:
            energy_label = "$E$ (eV)"

        colors = matplotlib.rcParams["axes.prop_cycle"].by_key()["color"]
        ncolors = len(colors)

        if ax is None:
            _, ax = plt.subplots()

        allpts = []
        keys = self._cohps.keys()
        for i, key in enumerate(keys):
            energies = self._cohps[key]["energies"]
            if not integrated:
                populations = self._cohps[key]["COHP"]
            else:
                populations = self._cohps[key]["ICOHP"]
            for spin in [Spin.up, Spin.down]:
                if spin in populations:
                    if invert_axes:
                        x = -populations[spin] if plot_negative else populations[spin]
                        y = energies
                        x = self._broaden(y, x, sigma=sigma)
                    else:
                        x = energies
                        y = -populations[spin] if plot_negative else populations[spin]
                        y = self._broaden(x, y, sigma=sigma)
                    allpts.extend(list(zip(x, y)))
                    if spin == Spin.up:
                        ax.plot(
                            x,
                            y,
                            color=colors[i % ncolors],
                            linestyle="-",
                            label=str(key),
                        )
                    else:
                        ax.plot(x, y, color=colors[i % ncolors], linestyle="--")

        if xlim:
            ax.set_xlim(xlim)
        xlim = ax.get_xlim()
        assert isinstance(xlim, tuple)

        if ylim:
            ax.set_ylim(ylim)
        else:
            relevanty = [p[1] for p in allpts if xlim[0] < p[0] < xlim[1]]
            plt.ylim((min(relevanty), max(relevanty)))

        grid_like_line_kwargs = {
            "color": matplotlib.rcParams["grid.color"],
            "linewidth": matplotlib.rcParams["grid.linewidth"],
            "linestyle": matplotlib.rcParams["grid.linestyle"],
            "alpha": matplotlib.rcParams["grid.alpha"],
            "zorder": 0,
        }

        if not invert_axes:
            ax.axhline(**grid_like_line_kwargs)

            if self.zero_at_efermi:
                ax.axvline(**grid_like_line_kwargs)

            else:
                ax.axvline(self._cohps[key]["efermi"], **grid_like_line_kwargs)
        else:
            ax.axvline(**grid_like_line_kwargs)

            if self.zero_at_efermi:
                ax.axhline(**grid_like_line_kwargs)
            else:
                ax.axhline(self._cohps[key]["efermi"], **grid_like_line_kwargs)

        if invert_axes:
            plt.xlabel(cohp_label)
            plt.ylabel(energy_label)
        else:
            plt.xlabel(energy_label)
            plt.ylabel(cohp_label)

        _ = ax.legend()
        return plt

    @staticmethod
    def _broaden(energies: np.ndarray, population: np.ndarray, sigma=None, cutoff=4.0):
        """Broaden the spectrum with a given standard deviation

        The population is convolved with a normalised Gaussian kernel. This
        requires the energy grid to be regularly-spaced.

        Args:
            energies: Regularly-spaced energy series
            population: Population data for broadening
            sigma: Standard deviation for Gaussian broadening. If sigma is None
                then the input data is returned without any processing.
            cutoff: Range cutoff for broadening kernel, as a multiple of sigma.

        Return:
            Broadened population
        """
        from scipy.signal import convolve
        from scipy.stats import norm

        if sigma is None:
            return population

        spacing = np.mean(np.diff(energies))
        if not np.allclose(np.diff(energies), spacing, atol=1e-5):
            raise ValueError(
                "Energy grid is not regular, cannot broaden with "
                "discrete convolution."
            )

        # Obtain symmetric mesh for broadening kernel, centered on zero
        kernel_x = np.arange(0, cutoff * sigma + 0.5 * spacing, spacing)
        kernel_x = np.concatenate([-kernel_x[-1:1:-1], kernel_x])

        kernel = norm.pdf(kernel_x, scale=sigma)

        return convolve(population, kernel, mode="same") / kernel.sum()



# Copyright (c) lobsterpy development team
# Distributed under the terms of a BSD 3-Clause "New" or "Revised" License

"""
Script to analyze Lobster outputs from the command line
"""
from __future__ import annotations

import argparse
import json
from math import log, sqrt
from pathlib import Path

import matplotlib.style
from pymatgen.electronic_structure.cohp import CompleteCohp

from lobsterpy.cohp.analyze import Analysis
from lobsterpy.cohp.describe import Description
from lobsterpy.plotting import PlainCohpPlotter, get_style_list


def main() -> None:
    """Entry point for setup.py installer"""
    args = get_parser().parse_args()
    run(args)


def get_parser() -> argparse.ArgumentParser:
    """Construct argumentparser with subcommands and sections"""
    parser = argparse.ArgumentParser(
        description="Analyze and plot results from Lobster runs."
    )

    # Arguments common to all actions
    input_parent = argparse.ArgumentParser(add_help=False)
    input_file_group = input_parent.add_argument_group("Input files")
    input_file_group.add_argument(
        "--poscar",
        "--POSCAR",
        dest="poscar",
        default="POSCAR",
        type=Path,
        help='path to POSCAR. Default is "POSCAR"',
    )
    input_file_group.add_argument(
        "--charge",
        default="CHARGE.lobster",
        type=Path,
        help='path to Charge.lobster. Default is "CHARGE.lobster"',
    )
    input_file_group.add_argument(
        "--icohplist",
        default="ICOHPLIST.lobster",
        type=Path,
        help='path to ICOHPLIST.lobster. Default is "ICOHPLIST.lobster"',
    )
    input_file_group.add_argument(
        "--cohpcar",
        default="COHPCAR.lobster",
        type=Path,
        help=(
            'path to COHPCAR.lobster. Default is "COHPCAR.lobster". This argument '
            "will also be read when COBICARs or COOPCARs are plotted."
        ),
    )
    input_file_group.add_argument(
        "--incar",
        default="INCAR",
        type=Path,
        help=('path to INCAR. Default is "INCAR".'),
    )
    input_file_group.add_argument(
        "--potcar",
        default="POTCAR",
        type=Path,
        help=('path to POTCAR. Default is "POTCAR".'),
    )

    output_parent = argparse.ArgumentParser(add_help=False)
    output_file_group = output_parent.add_argument_group("Output files")
    output_file_group.add_argument(
        "--incar-out",
        "--incarout",
        dest="incarout",
        default="INCAR.lobsterpy",
        type=Path,
        help='path to INCAR that will be generated by lobsterpy. Default is "INCAR.lobsterpy"',
    )
    output_file_group.add_argument(
        "--lobsterin-out",
        "--lobsterinout",
        dest="lobsterinout",
        default="lobsterin.lobsterpy",
        type=Path,
        help='base for path to lobsterins that will be generated by lobsterpy. Default is "lobsterin.lobsterpy"',
    )
    output_file_group.add_argument(
        "--overwrite",
        "--overwrite-files",
        dest="overwrite",
        default=False,
        action="store_true",
        help="overwrites already created INCARs an lobsterins with the give name.",
    )
    # TODO: Add some output arguments: options to supply your own basis
    output_file_group.add_argument(
        "--userbasis",
        "--user-basis",
        default=None,
        type=_element_basis,
        nargs="+",
        help="This setting will rely on a specific basis provided by the user "
        "(e.g.,  --userbasis Cr.3d.3p.4s N.2s.2p). Default is None.",
    )

    plotting_parent = argparse.ArgumentParser(add_help=False)
    plotting_group = plotting_parent.add_argument_group("Plotting")

    broadening_group = plotting_group.add_mutually_exclusive_group()
    broadening_group.add_argument_group("Broadening")
    broadening_group.add_argument(
        "--sigma",
        type=float,
        default=None,
        help="Standard deviation of Gaussian broadening.",
    )

    broadening_group.add_argument(
        "--fwhm",
        type=float,
        default=None,
        help="Full-width-half-maximum of Gaussian broadening.",
    )

    plotting_group.add_argument(
        "--integrated",
        action="store_true",
        help="Show integrated cohp/cobi/coop plots.",
    )

    plotting_group.add_argument(
        "--ylim",
        dest="ylim",
        nargs=2,
        default=None,
        type=float,
        help="Energy range for plots",
    )
    plotting_group.add_argument(
        "--xlim",
        dest="xlim",
        nargs=2,
        default=None,
        type=float,
        help="COHP/COBI/COOP range for plots",
    )

    plotting_group.add_argument(
        "--style",
        type=str,
        nargs="+",
        default=None,
        help="Matplotlib style sheet(s) for plot appearance",
    )
    plotting_group.add_argument(
        "--no-base-style",
        "--nobasestyle",
        action="store_true",
        dest="no_base_style",
        help=(
            "Disable inbuilt style entirely. This may prevent interference with external "
            "stylesheets when using --style."
        ),
    )
    plotting_group.add_argument("--title", type=str, default="", help="Plot title")
    plotting_group.add_argument(
        "--save-plot",
        "--saveplot",
        "-s",
        type=Path,
        metavar="FILENAME",
        default=None,
        dest="save_plot",
        help="Save plot to file",
    )
    plotting_group.add_argument(
        "--width", type=float, default=None, help="Plot width in inches"
    )
    plotting_group.add_argument(
        "--height", type=float, default=None, help="Plot height in inches"
    )
    plotting_group.add_argument(
        "--fontsize", "--font-size", type=float, default=None, help="Base font size"
    )

    auto_parent = argparse.ArgumentParser(add_help=False)
    auto_group = auto_parent.add_argument_group("Automatic analysis")
    auto_group.add_argument(
        "--json",
        nargs="?",
        type=Path,
        default=None,
        metavar="FILENAME",
        const=Path("lobsterpy.json"),
        help="Write a JSON file with the most important information",
    )
    auto_group.add_argument(
        "--allbonds",
        "--all-bonds",
        action="store_true",
        default=False,
        help="This option will force the automatc analysis to consider"
        " all bonds, not only cation-anion bonds (default) ",
    )

    subparsers = parser.add_subparsers(
        dest="action",
        required=True,
        help="Use -h/--help after the chosen subcommand to see further options.",
    )
    subparsers.add_parser(
        "description",
        parents=[input_parent, auto_parent],
        help=(
            "Deliver a text description of the COHP results from Lobster "
            "and VASP. Implementation of COBIs and COOPs will follow."
        ),
    )

    subparsers.add_parser(
        "automatic-plot",
        aliases=["automaticplot"],
        parents=[input_parent, auto_parent, plotting_parent],
        help=(
            "Plot most important COHPs automatically. Implementation "
            "of COBIs and COOPs will follow. This option also includes an automatic description."
        ),
    )

    subparsers.add_parser(
        "create-inputs",
        aliases=["createinputs"],
        parents=[input_parent, output_parent],
        help=(
            "Will create inputs for lobster computation. It only works with PBE POTCARs."
        ),
    )

    # Mode for normal plotting (without automatic detection of relevant COHPs)
    plot_parser = subparsers.add_parser(
        "plot",
        parents=[input_parent, plotting_parent],
        help="Plot specific COHPs/COBIs/COOPs based on bond numbers.",
    )

    plot_parser.add_argument(
        "bond_numbers",
        nargs="+",
        type=int,
        help="List of bond numbers, determining COHPs/COBIs/COOPs to include in plot.",
    )
    plot_coops_cobis = plot_parser.add_mutually_exclusive_group()
    plot_coops_cobis.add_argument(
        "--cobis",
        "--cobi",
        action="store_true",
        help="Plot COBIs",
    )
    plot_coops_cobis.add_argument(
        "--coops",
        "--coop",
        action="store_true",
        help="Plot COOPs",
    )
    plot_grouping = plot_parser.add_mutually_exclusive_group()
    plot_grouping.add_argument(
        "--summed",
        action="store_true",
        help="Show a summed COHP",
    )
    plot_grouping.add_argument(
        "--orbitalwise",
        dest="orbitalwise",
        nargs="+",
        default=None,
        type=str,
        help=(
            "Plot cohps of specific orbitals. e.g. to plot 2s-2s interaction of "
            'bond with label 1, use "lobsterpy plot 1 --orbitalwise 2s-2s". '
            'To plot all orbitalwise cohps of one bond, you can use "all" instead of "2s-2s". '
            "To plot orbitalwise interactions of more than one bond, use, for example, "
            '"lobsterpy plot 1 1 --orbitalwise "3s-3s" "2px-3s"'
        ),
    )
    return parser


def _element_basis(string: str):
    """
    Parse element and basis from string
    Args: str
    Returns: element, basis
    """
    cut_list = string.split(".")
    element = cut_list[0]
    basis = " ".join(cut_list[1:])
    return element, basis


def _user_figsize(width, height, aspect=None):
    """Get figsize options from user input, if any

    If only width x or height is provided, use a target aspect ratio to derive
    the other one.

    Returns a dict which can be merged into style kwargs
    """
    if width is None and height is None:
        return {}
    if width is not None and height is not None:
        return {"figure.figsize": (width, height)}

    if aspect is None:
        aspect = (sqrt(5) + 1) / 2  # Golden ratio
    if width is None:
        return {"figure.figsize": (height * aspect, height)}
    return {"figure.figsize": (width, width / aspect)}


# TODO: add automatic functionality for COBIs, COOPs
def run(args):
    """

    Args:
        args: args for cli

    """
    if args.action == "automaticplot":
        args.action = "automatic-plot"

    if args.action in ["description", "automatic-plot"]:
        if args.allbonds:
            whichbonds = "all"
        else:
            whichbonds = "cation-anion"
        analyse = Analysis(
            path_to_poscar=args.poscar,
            path_to_charge=args.charge,
            path_to_cohpcar=args.cohpcar,
            path_to_icohplist=args.icohplist,
            whichbonds=whichbonds,
        )

        describe = Description(analysis_object=analyse)
        describe.write_description()

        if args.json is not None:
            analysedict = analyse.condensed_bonding_analysis
            with open(args.json, "w") as fd:
                json.dump(analysedict, fd)

    if args.action in ["plot", "automatic-plot"]:
        style_kwargs = {}
        style_kwargs.update(_user_figsize(args.width, args.height))
        if args.fontsize:
            style_kwargs.update({"font.size": args.fontsize})

        style_list = get_style_list(
            no_base_style=args.no_base_style, styles=args.style, **style_kwargs
        )
        matplotlib.style.use(style_list)

        if args.sigma:
            sigma = args.sigma
        elif args.fwhm:
            sigma = args.fwhm / (2 * sqrt(2 * log(2)))
        else:
            sigma = None

    if args.action in ["automatic-plot"]:
        describe.plot_cohps(
            ylim=args.ylim,
            xlim=args.xlim,
            integrated=args.integrated,
            save=args.save_plot is not None,
            filename=args.save_plot,
            title=args.title,
            sigma=sigma,
        )

    if args.action == "plot":
        if args.cobis:
            filename = args.cohpcar.parent / "COBICAR.lobster"
            options = {"are_cobis": True, "are_coops": False}
        elif args.coops:
            filename = args.cohpcar.parent / "COOPCAR.lobster"
            options = {"are_cobis": False, "are_coops": True}
        else:
            filename = args.cohpcar
            options = {"are_cobis": False, "are_coops": False}

        completecohp = CompleteCohp.from_file(
            fmt="LOBSTER", filename=filename, structure_file=args.poscar, **options
        )
        cp = PlainCohpPlotter(**options)

        if not args.summed:
            # TODO: add checks for label in allewod labels -> print all labels
            # TODO: add check if args.oribtalwise is exactly as long as labels
            # TODO: add check if orbital is in args.orbitalwise

            for label in args.bond_numbers:
                if str(label) not in completecohp.bonds.keys():
                    raise IndexError(
                        "The provided bond label "
                        + str(label)
                        + " is not available in ICO**LIST.lobster.\n "
                        "Allowed options are in this list: \n"
                        + str([int(listi) for listi in list(completecohp.bonds.keys())])
                    )

            if not args.orbitalwise:
                for label in args.bond_numbers:
                    cp.add_cohp(label, completecohp.get_cohp_by_label(label=str(label)))
            else:
                if len(args.bond_numbers) != len(args.orbitalwise):
                    raise IndexError(
                        "Please provide as mainy orbitals as bond labels,"
                        " e.g., lobsterpy plot 1 1 --orbitalwise '2s-2s' '2s-2px'"
                    )

                for ilabel, label in enumerate(args.bond_numbers):
                    orbitals = args.orbitalwise[ilabel]

                    availableorbitals = list(
                        completecohp.orb_res_cohp[str(label)].keys()
                    )
                    orbitaloptions = [*availableorbitals, "all"]

                    if orbitals not in orbitaloptions:
                        raise IndexError(
                            "Orbital in not available for current bond. \n"
                            "For bond "
                            + str(label)
                            + " only the following orbital options are available: \n"
                            + str(orbitaloptions)
                        )

                    if orbitals != "all":
                        cp.add_cohp(
                            str(label) + ": " + orbitals,
                            completecohp.get_orbital_resolved_cohp(
                                label=str(label), orbitals=orbitals
                            ),
                        )
                    else:
                        for orbitals in availableorbitals:
                            cp.add_cohp(
                                str(label) + ": " + orbitals,
                                completecohp.get_orbital_resolved_cohp(
                                    label=str(label), orbitals=orbitals
                                ),
                            )
        else:
            cp.add_cohp(
                str(args.bond_numbers),
                completecohp.get_summed_cohp_by_label_list(
                    label_list=[str(label) for label in args.bond_numbers]
                ),
            )

        plt = cp.get_plot(
            integrated=args.integrated, xlim=args.xlim, ylim=args.ylim, sigma=sigma
        )

        ax = plt.gca()
        ax.set_title(args.title)

        if args.save_plot is None:
            plt.show()
        else:
            fig = plt.gcf()
            fig.savefig(args.save_plot)
    if args.action == "create-inputs":
        from pymatgen.core.structure import Structure
        from pymatgen.io.lobster import Lobsterin

        if args.userbasis is None:
            # This will rely on standard basis files as stored in pymatgen

            potcar_names = Lobsterin._get_potcar_symbols(POTCAR_input=args.potcar)

            list_basis_dict = Lobsterin.get_all_possible_basis_functions(
                structure=Structure.from_file(args.poscar), potcar_symbols=potcar_names
            )

            for ibasis, basis_dict in enumerate(list_basis_dict):
                lobsterinput = Lobsterin.standard_calculations_from_vasp_files(
                    args.poscar,
                    args.incar,
                    None,
                    option="standard",
                    dict_for_basis=basis_dict,
                )

                lobsterin_path = Path(str(args.lobsterinout) + "-" + str(ibasis))
                incar_path = Path(str(args.incarout) + "-" + str(ibasis))

                if (not lobsterin_path.is_file() and not incar_path.is_file()) or (
                    args.overwrite
                ):
                    lobsterinput.write_lobsterin(lobsterin_path)
                    lobsterinput.write_INCAR(
                        incar_input=args.incar,
                        incar_output=incar_path,
                        poscar_input=args.poscar,
                        isym=0,
                    )
                else:
                    raise ValueError(
                        'please use "--overwrite" if you would like to overwrite existing lobster inputs'
                    )
        else:
            # convert list userbasis to dict
            userbasis = {}
            for userbasis_single in args.userbasis:
                userbasis[userbasis_single[0]] = userbasis_single[1]

            lobsterinput = Lobsterin.standard_calculations_from_vasp_files(
                args.poscar,
                args.incar,
                None,
                option="standard",
                dict_for_basis=userbasis,
            )

            lobsterin_path = Path(str(args.lobsterinout) + "-" + str(0))
            incar_path = Path(str(args.incarout) + "-" + str(0))

            if (not lobsterin_path.is_file() and not incar_path.is_file()) or (
                args.overwrite
            ):
                lobsterinput.write_lobsterin(lobsterin_path)
                lobsterinput.write_INCAR(
                    incar_input=args.incar,
                    incar_output=incar_path,
                    poscar_input=args.poscar,
                    isym=0,
                )
            else:
                raise ValueError(
                    'please use "--overwrite" if you would like to overwrite existing lobster inputs'
                )


if __name__ == "__main__":
    main()



from __future__ import annotations

import unittest
from pathlib import Path

from lobsterpy.cohp.analyze import Analysis

CurrentDir = Path(__file__).absolute().parent
TestDir = CurrentDir / "../../"


class TestAnalyse(unittest.TestCase):
    def setUp(self):
        self.analyse_NaCl = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_NaCl_comp_range = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl_comp_range/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/NaCl_comp_range/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/NaCl_comp_range/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/NaCl_comp_range/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_NaCl_nan = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            start=-4.0,
        )

        self.analyse_NaCl_valences = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=None,
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_NaCl_madelung = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            path_to_madelung=TestDir / "TestData/NaCl/MadelungEnergies.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_BaTiO3 = Analysis(
            path_to_poscar=TestDir / "TestData/BaTiO3/POSCAR",
            path_to_cohpcar=TestDir / "TestData/BaTiO3/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/BaTiO3/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/BaTiO3/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_BaTaO2N1 = Analysis(
            path_to_poscar=TestDir / "TestData/BaTaO2N1/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/BaTaO2N1/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/BaTaO2N1/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/BaTaO2N1/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_BaTiO3_differentcutoff = Analysis(
            path_to_poscar=TestDir / "TestData/BaTiO3/POSCAR",
            path_to_cohpcar=TestDir / "TestData/BaTiO3/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/BaTiO3/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/BaTiO3/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.001,
        )

        self.analyse_NaCl_distorted = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl_distorted/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl_distorted/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl_distorted/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl_distorted/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_NaCl_spin = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl_spin/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl_spin/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl_spin/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl_spin/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_NaCl_all = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            whichbonds="all",
            cutoff_icohp=0.1,
        )

        self.analyse_NaCl_madelung_all = Analysis(
            path_to_poscar=TestDir / "TestData/NaCl/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaCl/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaCl/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaCl/CHARGE.lobster",
            path_to_madelung=TestDir / "TestData/NaCl/MadelungEnergies.lobster",
            whichbonds="all",
            cutoff_icohp=0.1,
        )
        self.analyse_NaSi_madelung_all = Analysis(
            path_to_poscar=TestDir / "TestData/NaSi/POSCAR",
            path_to_cohpcar=TestDir / "TestData/NaSi/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/NaSi/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/NaSi/CHARGE.lobster",
            path_to_madelung=TestDir / "TestData/NaSi/MadelungEnergies.lobster",
            whichbonds="all",
            cutoff_icohp=0.1,
        )

        self.analyse_BaTaO2N1_cutoff = Analysis(
            path_to_poscar=TestDir / "TestData/BaTaO2N1/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/BaTaO2N1/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/BaTaO2N1/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/BaTaO2N1/CHARGE.lobster.gz",
            whichbonds="all",
            cutoff_icohp=0.001,
        )

        self.analyse_NaSbF6 = Analysis(
            path_to_poscar=TestDir / "TestData/NaSbF6/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/NaSbF6/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/NaSbF6/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/NaSbF6/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        self.analyse_NaSbF6_anbd = Analysis(
            path_to_poscar=TestDir / "TestData/NaSbF6/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/NaSbF6/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/NaSbF6/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/NaSbF6/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            start=-5.5,
        )

        self.analyse_CdF = Analysis(
            path_to_poscar=TestDir / "TestData/CdF/POSCAR",
            path_to_cohpcar=TestDir / "TestData/CdF/COHPCAR.lobster",
            path_to_icohplist=TestDir / "TestData/CdF/ICOHPLIST.lobster",
            path_to_charge=TestDir / "TestData/CdF/CHARGE.lobster",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
            start=-4.0,
        )

        self.analyse_CdF_comp_range = Analysis(
            path_to_poscar=TestDir / "TestData/CdF_comp_range/POSCAR.gz",
            path_to_cohpcar=TestDir / "TestData/CdF_comp_range/COHPCAR.lobster.gz",
            path_to_icohplist=TestDir / "TestData/CdF_comp_range/ICOHPLIST.lobster.gz",
            path_to_charge=TestDir / "TestData/CdF_comp_range/CHARGE.lobster.gz",
            whichbonds="cation-anion",
            cutoff_icohp=0.1,
        )

        # different environment than O:6

    def test_exception(self):
        with self.assertRaises(ValueError):
            self.analyse_BaTaO2N1 = Analysis(
                path_to_poscar=TestDir / "TestData/BaTaO2N1/POSCAR.gz",
                path_to_cohpcar=TestDir / "TestData/BaTaO2N1/COHPCAR.lobster.gz",
                path_to_icohplist=TestDir / "TestData/BaTaO2N1/ICOHPLIST.lobster.gz",
                path_to_charge=TestDir / "TestData/BaTaO2N1/CHARGE.lobster.gz",
                whichbonds="cation-cation",
                cutoff_icohp=0.1,
            )
        with self.assertRaises(ValueError) as err:
            self.analyse_C = Analysis(
                path_to_poscar=TestDir / "TestData/C/POSCAR",
                path_to_cohpcar=TestDir / "TestData/C/COHPCAR.lobster",
                path_to_icohplist=TestDir / "TestData/C/ICOHPLIST.lobster",
                path_to_charge=TestDir / "TestData/C/CHARGE.lobster",
                whichbonds="cation-anion",
                cutoff_icohp=0.1,
            )
        self.assertEqual(
            err.exception.__str__(),
            "Consider switching to an analysis of all bonds and not only cation-anion bonds. It looks like no cations are detected.",
        )

    def test_all_attributes_NaCl_Mulliken(self):
        self.assertEqual(
            self.analyse_NaCl.condensed_bonding_analysis["formula"], "NaCl"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl.condensed_bonding_analysis["max_considered_bond_length"],
            5.69169,
        )
        self.assertAlmostEqual(
            self.analyse_NaCl.condensed_bonding_analysis["number_of_considered_ions"], 1
        )
        self.assertEqual(
            self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["env"], "O:6"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                    "ICOHP_mean"
                ]
            ),
            -0.57,
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                    "ICOHP_sum"
                ]
            ),
            -3.39,
        )
        self.assertEqual(
            self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "has_antibdg_states_below_Efermi"
            ],
            True,
        )
        self.assertEqual(
            self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "number_of_bonds"
            ],
            6,
        )
        self.assertEqual(
            self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["ion"], "Na"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["charge"], 0.78
        )
        self.assertListEqual(
            self.analyse_NaCl.condensed_bonding_analysis["sites"][0]["relevant_bonds"],
            ["21", "23", "24", "27", "28", "30"],
        )
        self.assertEqual(
            self.analyse_NaCl.condensed_bonding_analysis["type_charges"], "Mulliken"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_madelung.condensed_bonding_analysis["madelung_energy"],
            -5.40,
        )

    def test_all_attributes_NaCl_valences(self):
        self.assertEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["formula"], "NaCl"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis[
                "max_considered_bond_length"
            ],
            5.69169,
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis[
                "number_of_considered_ions"
            ],
            1,
        )
        self.assertEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0]["env"],
            "O:6",
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0][
                    "bonds"
                ]["Cl"]["ICOHP_mean"]
            ),
            -0.57,
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0][
                    "bonds"
                ]["Cl"]["ICOHP_sum"]
            ),
            -3.39,
        )
        self.assertEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0]["bonds"][
                "Cl"
            ]["has_antibdg_states_below_Efermi"],
            True,
        )
        self.assertEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0]["bonds"][
                "Cl"
            ]["number_of_bonds"],
            6,
        )
        self.assertEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0]["ion"],
            "Na",
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0]["charge"],
            1,
        )
        self.assertListEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["sites"][0][
                "relevant_bonds"
            ],
            ["21", "23", "24", "27", "28", "30"],
        )
        self.assertEqual(
            self.analyse_NaCl_valences.condensed_bonding_analysis["type_charges"],
            "Valences",
        )

    def test_all_attributes_BaTiO3(self):
        self.assertEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["formula"], "BaTiO3"
        )
        self.assertAlmostEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis[
                "max_considered_bond_length"
            ],
            5.94882,
        )
        self.assertAlmostEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["number_of_considered_ions"],
            1,
        )
        self.assertEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1]["env"], "O:6"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1]["bonds"][
                    "O"
                ]["ICOHP_sum"]
            ),
            -21.24,
        )
        self.assertEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1]["bonds"]["O"][
                "has_antibdg_states_below_Efermi"
            ],
            True,
        )
        self.assertEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1]["bonds"]["O"][
                "number_of_bonds"
            ],
            6,
        )
        self.assertEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1]["ion"], "Ti"
        )
        self.assertAlmostEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1]["charge"], 0.96
        )
        self.assertListEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["sites"][1][
                "relevant_bonds"
            ],
            ["87", "88", "98", "101", "109", "114"],
        )
        self.assertEqual(
            self.analyse_BaTiO3.condensed_bonding_analysis["type_charges"], "Mulliken"
        )

    def test_all_attributes_NaCl_all(self):
        print(self.analyse_NaCl_all.condensed_bonding_analysis)
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["formula"], "NaCl"
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["formula"], "NaCl"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis[
                "max_considered_bond_length"
            ],
            5.69169,
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis[
                "number_of_considered_ions"
            ],
            2,
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["env"], "O:6"
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][1]["env"], "O:6"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["bonds"][
                    "Cl"
                ]["ICOHP_mean"]
            ),
            -0.57,
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["bonds"][
                    "Cl"
                ]["ICOHP_sum"]
            ),
            -3.39,
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "has_antibdg_states_below_Efermi"
            ],
            True,
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "number_of_bonds"
            ],
            6,
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["ion"], "Na"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0]["charge"], 0.78
        )
        self.assertListEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["sites"][0][
                "relevant_bonds"
            ],
            ["21", "23", "24", "27", "28", "30"],
        )
        self.assertEqual(
            self.analyse_NaCl_all.condensed_bonding_analysis["type_charges"], "Mulliken"
        )

    def test_final_dicts(self):
        self.assertAlmostEqual(
            self.analyse_NaSi_madelung_all.final_dict_bonds["Na-Si"]["ICOHP_mean"],
            -0.44833333333,
        )
        self.assertDictEqual(self.analyse_NaCl.final_dict_ions["Na"], {"O:6": 1})
        self.assertEqual(
            self.analyse_NaCl.final_dict_bonds["Cl-Na"]["has_antbdg"], True
        )
        self.assertAlmostEqual(
            self.analyse_NaCl.final_dict_bonds["Cl-Na"]["ICOHP_mean"],
            -0.565000000000000,
        )

        self.assertDictEqual(self.analyse_BaTiO3.final_dict_ions["Ti"], {"O:6": 1})
        self.assertEqual(
            self.analyse_BaTiO3.final_dict_bonds["O-Ti"]["has_antbdg"], True
        )
        self.assertAlmostEqual(
            self.analyse_BaTiO3.final_dict_bonds["O-Ti"]["ICOHP_mean"],
            -3.5399999999999996,
        )

        self.assertDictEqual(
            self.analyse_NaCl_distorted.final_dict_ions, {"Na": {"O:6": 4}}
        )
        self.assertEqual(
            self.analyse_NaCl_distorted.final_dict_bonds["Cl-Na"]["has_antbdg"], True
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_distorted.final_dict_bonds["Cl-Na"]["ICOHP_mean"],
            -0.5658333333333334,
        )
        self.assertEqual(
            self.analyse_NaCl_spin.final_dict_bonds["Cl-Na"]["has_antbdg"], True
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_spin.final_dict_bonds["Cl-Na"]["ICOHP_mean"],
            -0.5650000000000001,
        )
        self.assertDictEqual(self.analyse_NaCl_spin.final_dict_ions, {"Na": {"O:6": 1}})
        self.assertAlmostEqual(
            self.analyse_NaCl_all.final_dict_bonds["Cl-Na"]["ICOHP_mean"],
            -0.5650000000000001,
        )
        self.assertDictEqual(
            self.analyse_NaCl_all.final_dict_ions, {"Na": {"O:6": 1}, "Cl": {"O:6": 1}}
        )

    def test_all_attributes_NaSbF6(self):
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["formula"], "NaSbF6"
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis[
                "max_considered_bond_length"
            ],
            5.98893,
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["number_of_considered_ions"],
            2,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["env"], "O:6"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"][
                    "F"
                ]["ICOHP_sum"]
            ),
            -3.65,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "has_antibdg_states_below_Efermi"
            ],
            True,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "number_of_bonds"
            ],
            6,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "bonding"
            ]["integral"],
            3.77,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "bonding"
            ]["perc"],
            0.95929,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "antibonding"
            ]["perc"],
            0.04071,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "antibonding"
            ]["integral"],
            0.16,
        )
        self.assertAlmostEqual(
            abs(
                float(
                    self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"][
                        "F"
                    ]["ICOHP_sum"]
                )
            ),
            (
                round(
                    self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["bonds"][
                        "F"
                    ]["bonding"]["integral"]
                    - self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0][
                        "bonds"
                    ]["F"]["antibonding"]["integral"],
                    2,
                )
            ),
            delta=0.10,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["ion"], "Na"
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0]["charge"], 0.91
        )
        self.assertListEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][0][
                "relevant_bonds"
            ],
            ["21", "25", "31", "34", "43", "47"],
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["env"], "O:6"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"][
                    "F"
                ]["ICOHP_sum"]
            ),
            -32.71,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"]["F"][
                "has_antibdg_states_below_Efermi"
            ],
            False,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"]["F"][
                "number_of_bonds"
            ],
            6,
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"]["F"][
                "bonding"
            ]["perc"],
            1.0,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"]["F"][
                "antibonding"
            ]["perc"],
            0.0,
        )
        self.assertAlmostEqual(
            abs(
                float(
                    self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"][
                        "F"
                    ]["ICOHP_sum"]
                )
            ),
            (
                round(
                    self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["bonds"][
                        "F"
                    ]["bonding"]["integral"]
                    - self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1][
                        "bonds"
                    ]["F"]["antibonding"]["integral"],
                    2,
                )
            ),
            delta=0.10,
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["ion"], "Sb"
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1]["charge"], 2.91
        )
        self.assertListEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["sites"][1][
                "relevant_bonds"
            ],
            ["63", "69", "73", "80", "81", "87"],
        )
        self.assertEqual(
            self.analyse_NaSbF6.condensed_bonding_analysis["type_charges"], "Mulliken"
        )

    def test_all_attributes_NaSbF6_anbd(self):
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["formula"], "NaSbF6"
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis[
                "max_considered_bond_length"
            ],
            5.98893,
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis[
                "number_of_considered_ions"
            ],
            2,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["env"],
            "O:6",
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0][
                    "bonds"
                ]["F"]["ICOHP_sum"]
            ),
            -3.65,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["bonds"][
                "F"
            ]["has_antibdg_states_below_Efermi"],
            True,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["bonds"][
                "F"
            ]["number_of_bonds"],
            6,
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["bonds"][
                "F"
            ]["bonding"]["perc"],
            0.0,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["bonds"][
                "F"
            ]["antibonding"]["perc"],
            0.0,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["ion"], "Na"
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0]["charge"],
            0.91,
        )
        self.assertListEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][0][
                "relevant_bonds"
            ],
            ["21", "25", "31", "34", "43", "47"],
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["env"],
            "O:6",
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1][
                    "bonds"
                ]["F"]["ICOHP_sum"]
            ),
            -32.71,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["bonds"][
                "F"
            ]["has_antibdg_states_below_Efermi"],
            False,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["bonds"][
                "F"
            ]["number_of_bonds"],
            6,
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["bonds"][
                "F"
            ]["bonding"]["perc"],
            1.0,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["bonds"][
                "F"
            ]["antibonding"]["perc"],
            0.0,
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["ion"], "Sb"
        )
        self.assertAlmostEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1]["charge"],
            2.91,
        )
        self.assertListEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["sites"][1][
                "relevant_bonds"
            ],
            ["63", "69", "73", "80", "81", "87"],
        )
        self.assertEqual(
            self.analyse_NaSbF6_anbd.condensed_bonding_analysis["type_charges"],
            "Mulliken",
        )

    def test_all_attributes_CdF(self):
        self.assertEqual(self.analyse_CdF.condensed_bonding_analysis["formula"], "CdF2")
        self.assertAlmostEqual(
            self.analyse_CdF.condensed_bonding_analysis["max_considered_bond_length"],
            5.99501,
        )
        self.assertAlmostEqual(
            self.analyse_CdF.condensed_bonding_analysis["number_of_considered_ions"],
            1,
        )
        self.assertEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["env"], "C:8"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_CdF.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                    "ICOHP_sum"
                ]
            ),
            -4.96,
        )
        self.assertEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "has_antibdg_states_below_Efermi"
            ],
            True,
        )
        self.assertEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "number_of_bonds"
            ],
            8,
        )
        self.assertAlmostEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "bonding"
            ]["perc"],
            0.0,
        )
        self.assertEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["bonds"]["F"][
                "antibonding"
            ]["perc"],
            1.0,
        )
        self.assertEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["ion"], "Cd"
        )
        self.assertAlmostEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["charge"], 1.57
        )
        self.assertListEqual(
            self.analyse_CdF.condensed_bonding_analysis["sites"][0]["relevant_bonds"],
            ["25", "32", "35", "36", "57", "58", "61", "68"],
        )
        self.assertEqual(
            self.analyse_CdF.condensed_bonding_analysis["type_charges"], "Mulliken"
        )

    def test_all_attributes_NaCl_nan(self):
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["formula"], "NaCl"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis[
                "max_considered_bond_length"
            ],
            5.69169,
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis[
                "number_of_considered_ions"
            ],
            1,
        )
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["env"], "O:6"
        )
        self.assertAlmostEqual(
            float(
                self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["bonds"][
                    "Cl"
                ]["ICOHP_sum"]
            ),
            -3.39,
        )
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "has_antibdg_states_below_Efermi"
            ],
            True,
        )
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "number_of_bonds"
            ],
            6,
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "bonding"
            ]["perc"],
            0.0,
        )
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["bonds"]["Cl"][
                "antibonding"
            ]["perc"],
            0.0,
        )
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["ion"], "Na"
        )
        self.assertAlmostEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0]["charge"], 0.78
        )
        self.assertListEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["sites"][0][
                "relevant_bonds"
            ],
            ["21", "23", "24", "27", "28", "30"],
        )
        self.assertEqual(
            self.analyse_NaCl_nan.condensed_bonding_analysis["type_charges"], "Mulliken"
        )

    def test_ICOHP_sum_NaCl(self):
        self.assertAlmostEqual(
            abs(
                float(
                    self.analyse_NaCl_comp_range.condensed_bonding_analysis["sites"][0][
                        "bonds"
                    ]["Cl"]["ICOHP_sum"]
                )
            ),
            (
                round(
                    self.analyse_NaCl_comp_range.condensed_bonding_analysis["sites"][0][
                        "bonds"
                    ]["Cl"]["bonding"]["integral"]
                    - self.analyse_NaCl_comp_range.condensed_bonding_analysis["sites"][
                        0
                    ]["bonds"]["Cl"]["antibonding"]["integral"],
                    2,
                )
            ),
            delta=0.10,
        )

    def test_ICOHP_sum_CdF(self):
        self.assertAlmostEqual(
            abs(
                float(
                    self.analyse_CdF_comp_range.condensed_bonding_analysis["sites"][0][
                        "bonds"
                    ]["F"]["ICOHP_sum"]
                )
            ),
            (
                round(
                    self.analyse_CdF_comp_range.condensed_bonding_analysis["sites"][0][
                        "bonds"
                    ]["F"]["bonding"]["integral"]
                    - self.analyse_CdF_comp_range.condensed_bonding_analysis["sites"][
                        0
                    ]["bonds"]["F"]["antibonding"]["integral"],
                    2,
                )
            ),
            delta=0.10,
        )


if __name__ == "__main__":
    unittest.main()
